unit engine_rtf;

interface

type
  TRTFFile = class
             private
               OutFile : Text;
               Cached : Boolean;
               procedure UnBBCodeString(var Str : String);
               procedure AddData(Str : String);
             public
               constructor Create(Filename : String; Font : String = 'Courier New'; Cache : Boolean = true);
               destructor Destroy; override;
               procedure AddLine(Line : String);
             end;

  TLogFile = class(TRTFFile)
             private
               procedure TrimCaller(var Caller : String);
             public
               procedure AddMessage(Caller, Msg : String);
               procedure AddWarning(Caller, Msg : String);
               procedure AddSuccess(Caller, Msg : String);
               procedure AddError(Caller, Msg : String; Level : Integer);
             end;

var
  LogFile : TLogFile;

implementation

uses
  SysUtils;

type
  TColor = (Black, White, Red, Green, Blue, Yellow, Gray, dkGray, Cyan, Magenta,
            dkRed, dkGreen, dkBlue, ltRed, ltGreen, ltBlue, Orange);
  TBBCodes = (Bold, uBold, Italic, uItalic, Underline, uUnderline,
              Strikeout, uStrikeout, Sub, uSub, Sup, uSup, Plain, uColor);

const
  Colors : Array[TColor] of Array[0..2] of Byte = ((0,0,0), (255,255,255),
             (255,0,0), (0,255,0), (0,0,255), (255,255,0), (127,127,127), (64,64,64),
             (0, 255, 255), (255,0,255), (127,0,0), (0,127,0),(0,0,127),
             (255,127,127),(127,255,127),(127,127,255),(255,127,0));

  ColorNames : Array[TColor] of ShortString = ('BLACK', 'WHITE', 'RED',
                'GREEN', 'BLUE', 'YELLOW', 'GRAY', 'DKGRAY', 'CYAN', 'MAGENTA',
                'DKRED', 'DKGREEN', 'DKBLUE', 'LTRED', 'LTGREEN', 'LTBLUE',
                'ORANGE');

  BBCodes : Array[TBBCodes] of Array[0..1] of ShortString =
                (
                  ('B', '\b'), ('/B', '\b0'),
                  ('I', '\i'), ('/I', '\i0'),
                  ('U', '\ul'), ('/U', '\ul0'),
                  ('S', '\strike'), ('/S', '\strike0'),
                  ('SUB', '\sub'), ('/SUB', '\sub0'),
                  ('SUP', '\super'), ('/SUP', '\super0'),
                  ('PLAIN', '\plain'), ('/C', '\cf0')
                );

  MESSAGE_COLOR = 'BLACK';
  WARNING_COLOR = 'ORANGE';
  ERROR_COLOR   = 'RED';
  CALLER_COLOR  = 'DKGRAY';
  SUCCESS_COLOR = 'DKGREEN';

{ TRTFFile }

procedure TRTFFile.AddData(Str: String);
begin
  Write(OutFile, Str);
  if not Cached then
    Flush(OutFile);
end;

procedure TRTFFile.AddLine(Line: String);
begin
  UnBBCodeString(Line);
  Line := '{\pard ' + Line + ' \par}'+#13#10;
  AddData(Line);
end;

constructor TRTFFile.Create(Filename, Font: String; Cache: Boolean);
var
  Color : TColor;
begin
  AssignFile(OutFile, Filename);
  ReWrite(OutFile);

  Cached := Cache;
  AddData(Format('{\rtf1\ansi\deff0{\fonttbl{\f0 %s;}}'+#13#10,[Font]));

  AddData('{\colortbl;');
  for Color := Low(Color) to High(Color) do
  begin
    AddData(Format('\red%d\green%d\blue%d;',
                     [Colors[Color][0],Colors[Color][1],Colors[Color][2]]));
  end;
  AddData('}'+#13#10);
end;

destructor TRTFFile.Destroy;
begin
  AddData('}');
  CloseFile(OutFile);
  inherited;
end;

procedure TRTFFile.UnBBCodeString(var Str: String);
var
  Color : TColor;
  BBCode : TBBCodes;
begin
  Str := StringReplace(Str, '\', '\\',
    [rfReplaceAll, rfIgnoreCase]);
  Str := StringReplace(Str, '{', '\{',
    [rfReplaceAll, rfIgnoreCase]);
  Str := StringReplace(Str, '}', '\}',
    [rfReplaceAll, rfIgnoreCase]);

  for Color := Low(Color) to High(Color) do
    Str := StringReplace(Str, '['+ ColorNames[Color] + ']', '\cf'+IntToStr(Ord(Color)+1),
      [rfReplaceAll, rfIgnoreCase]);

  for BBCode := Low(TBBCodes) to High(TBBCodes) do
  begin
    Str := StringReplace(Str, '['+ BBCodes[BBCode][0] + ']', BBCodes[BBCode][1],
      [rfReplaceAll, rfIgnoreCase]);
  end;
end;

{ TLogFile }

procedure TLogFile.AddError(Caller, Msg: String; Level: Integer);
begin
  TrimCaller(Caller);
  Msg := Format('[%s][B] %s[/B] : [%s][B] %s', [Caller_Color, Caller, Error_Color, Msg]);
  AddLine(Msg);
end;

procedure TLogFile.AddMessage(Caller, Msg: String);
begin
  TrimCaller(Caller);
  Msg := Format('[%s][B] %s[/B] : [%s] %s', [Caller_Color, Caller, Message_Color, Msg]);
  AddLine(Msg);
end;

procedure TLogFile.AddSuccess(Caller, Msg: String);
begin
  TrimCaller(Caller);
  Msg := Format('[%s][B] %s[/B] : [%s][B] %s', [Caller_Color, Caller, Success_Color, Msg]);
  AddLine(Msg);
end;

procedure TLogFile.AddWarning(Caller, Msg: String);
begin
  TrimCaller(Caller);
  Msg := Format('[%s][B] %s[/B] : [%s][B] %s', [Caller_Color, Caller, Warning_Color, Msg]);
  AddLine(Msg);
end;

procedure TLogFile.TrimCaller(var Caller: String);
const
  MinLen = 8;
var
  len : Integer;
begin
  len := Length(Caller);
  if len < MinLen then
  begin
    SetLength(Caller, MinLen);
    inc(len);

    while len <= MinLen do
    begin
      Caller[len] := ' ';
      inc(len);
    end;
  end;
end;

initialization
  LogFile := TLogFile.Create('log.rtf', 'Courier New', false);
  LogFile.AddSuccess('engine_rtf', 'initialized...');

finalization
  LogFile.AddSuccess('engine_rtf', 'Shutdown');
  if Assigned(LogFile) then LogFile.Destroy;
  LogFile := nil;

end.
