unit synth3_waveforms;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, fmod_synthutils, engine_rtf, ComCtrls, ExtCtrls,
  spectrum_analyzer, Math, ExtDlgs;

type
  TWaveGenForm = class(TForm)
    FreqBar: TScrollBar;
    FreqLabel: TLabel;
    SpectrumPanel: TPanel;
    CPUUsageLabel: TLabel;
    SpectrumCheck: TCheckBox;
    SpectrumTimer: TTimer;
    PulseLenBar: TScrollBar;
    PulseLenLabel: TLabel;
    WaveModeBar: TScrollBar;
    WaveModeLabel: TLabel;
    GraphBox: TPaintBox;
    SavePictureDialog: TSavePictureDialog;
    CPUUsage: TProgressBar;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpectrumTimerTimer(Sender: TObject);
    procedure SpectrumCheckClick(Sender: TObject);
    procedure PulseLenBarChange(Sender: TObject);
    procedure FreqBarChange(Sender: TObject);
    procedure WaveModeBarChange(Sender: TObject);
    procedure GraphBoxPaint(Sender: TObject);
    procedure GraphBoxDblClick(Sender: TObject);
  private
    { Private-Deklarationen }
    SoundActive : Boolean;
    Spectrum : TSpectrum;
    SaveImage : Boolean;
  public
    { Public-Deklarationen }

    function SinWave(x : Single): Single;
    function DreieckWave(x : Single): Single;
    function PulseWave(x : Single): Single;
    function SaegezahnWave(x : Single): Single;
    function RauschenWave(x : Single): Single;

    function fMod(x, divisor : Single): Single;

    function CalcWave(Mode : Integer; x : Single) : Single;
  end;

var
  WaveGenForm: TWaveGenForm;

implementation

{$R *.DFM}

uses
  fmod, fmodtypes, fmoderrors;

const
  SamplingRate = 44100;

var
  Stream: PFSoundStream;
  Channel : Integer;
  Pos : Integer;
  Freq : Integer;
  CPUUpdate : Integer;

function StreamCallback(Stream: PFSoundStream; Buff: Pointer; Len, Param: Integer): ByteBool; stdcall;
var
  Count: Integer;
  Buffer: PChar;
  CPUUsage : Single;
  Value : Single;
  Delta : Single;
  Mode : Integer;

begin
  Freq := WaveGenForm.FreqBar.Position;
  Buffer := PChar(Buff);
  Count := 0;
  Mode := WaveGenForm.WaveModeBar.Position;

  while Count < (Len shr 2) do		{ >>2 = 16bit stereo (4 bytes per sample) }
  begin
    Value := WaveGenForm.CalcWave(Mode, 1/SamplingRate*Freq*Pos);

    PSmallInt(Buffer)^ := Trunc(Value * (32767.0/2.0));    { Left channel }
    Inc(Buffer, 2);
    PSmallInt(Buffer)^ := Trunc(Value * (32767.0/2.0));   { Right channel }
    Inc(Buffer, 2);

    Inc(Count);
    Inc(Pos);
  end;

  Inc(CPUUpdate);
  if CPUUpdate > 10 then
  begin
    CPUUsage := FSOUND_GetCPUUsage;
    WaveGenForm.CPUUsage.Position := Round(CPUUsage*10);
    WaveGenForm.CPUUsageLabel.Caption := Format('CPU-Last: %f %%', [CPUUsage]);
    CPUUpdate := 0;
  end;

  Result := True;
end;


procedure TWaveGenForm.FormCreate(Sender: TObject);
begin
  SaveImage := false;
  Pos := 0;
  CPUUpdate := 10;
  FreqBarChange(Self);
  PulseLenBarChange(Self);
  Randomize;

  SoundActive := OpenFMOD;
  if SoundActive then
  begin
    if not FSOUND_Init(SamplingRate, 16, 0) then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der Initialisierung von FMOD ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;

    Stream := FSOUND_Stream_Create(StreamCallback, 2500 * 2 * 2, FSOUND_NORMAL or FSOUND_16BITS or FSOUND_STEREO, SamplingRate, 12345);
    if Stream = nil then
    begin
      LogFile.AddWarning('->FormCreate',
        Format('Bei der des Sound-Streams ist folgender Fehler aufgetreten: %s',
               [FMOD_ErrorString(FSOUND_GetError)]));
      SoundActive := false;
      Exit;
    end;
    Channel := FSOUND_Stream_Play(FSOUND_FREE, Stream);

    Spectrum := TSpectrum.Create(SpectrumPanel);
    Spectrum.Parent := SpectrumPanel;
    Spectrum.Left := 1;
    Spectrum.Top := 1;
    Spectrum.Height := SpectrumPanel.Height -1;
    Spectrum.Visible := true;
    Spectrum.SpectrumEnabled := SpectrumCheck.Checked;
  end
  else
    LogFile.AddWarning('->FormCreate', 'Soundausgabe nicht mglich!');
end;

procedure TWaveGenForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if Assigned(Spectrum) then
  begin
    Spectrum.Free;
    Spectrum := nil
  end;

  if SoundActive then
    FSOUND_Stream_Close(Stream);
  CloseFMOD;
end;

procedure TWaveGenForm.SpectrumTimerTimer(Sender: TObject);
begin
  if Assigned(Spectrum) and (Spectrum.SpectrumEnabled) then
    Spectrum.RePaint;
end;

procedure TWaveGenForm.SpectrumCheckClick(Sender: TObject);
begin
  Spectrum.SpectrumEnabled := SpectrumCheck.Checked;
end;

function TWaveGenForm.DreieckWave(x: Single): Single;
begin
  Result := Abs((fMod(x, 1.0) - 0.5) * 2*2)-1;
end;

function TWaveGenForm.fMod(x, divisor: Single): Single;
begin
  Result := Trunc(x / divisor);
  Result := x - Result * divisor;
end;

function TWaveGenForm.PulseWave(x: Single): Single;
  function Sgn(x : Single) : Single;
  begin
    if x < 0 then
      Result := -1.0
    else
      Result := +1.0;
  end;
begin
  Result := fMod(x, 1);
  Result := Sgn(Result - 1/1000 * PulseLenBar.Position);
end;

function TWaveGenForm.SaegezahnWave(x: Single): Single;
begin
  Result := fMod(x, 1);
  Result := (-1 + 2 * Result);
end;

function TWaveGenForm.SinWave(x: Single): Single;
begin
  Result := Sin(2*Pi*x);
end;

function TWaveGenForm.RauschenWave(x: Single): Single;
begin
  Result := (Random -0.5)*2;
end;

function TWaveGenForm.CalcWave(Mode: Integer; x: Single): Single;
begin
  case Mode of
    0: Result := SinWave(x);
    1: Result := DreieckWave(x);
    2: Result := SaegezahnWave(x);
    3: Result := PulseWave(x);
    4: Result := RauschenWave(x);
  else
    Result := 0.0;
  end;
end;

procedure TWaveGenForm.PulseLenBarChange(Sender: TObject);
begin
  PulseLenLabel.Caption := Format('Pulse-Lnge: %f %%', [PulseLenBar.Position / 10]);
  GraphBox.Refresh;
end;

procedure TWaveGenForm.FreqBarChange(Sender: TObject);
var
  Freq : Integer;
begin
  Freq := FreqBar.Position;
  if Freq < 1000 then
    FreqLabel.Caption := Format('%d Hz Lambda = %f m', [Freq, 331/Freq])
  else
    FreqLabel.Caption := Format('%f kHz Lambda = %f mm', [Freq/1000, 331/Freq*1000])
end;

procedure TWaveGenForm.WaveModeBarChange(Sender: TObject);
const
  WaveModes : Array[0..4] of String = ('Sinus', 'Dreieck', 'Sgezahn', 'Puls', 'Rauschen');
begin
  if (WaveModeBar.Position >= 0) and (WaveModeBar.Position <= High(WaveModes)) then
    WaveModeLabel.Caption := 'Wellenart: ' + WaveModes[WaveModeBar.Position]
  else
    WaveModeLabel.Caption := 'Wellenart: undefiniert';
  GraphBox.Refresh;
end;

procedure TWaveGenForm.GraphBoxPaint(Sender: TObject);
var
  Buffer : TBitMap;
  UnitLen, UnitLenTel : Single;

  x, max,y : Integer;
  ys, Midline : Single;
  Mode : Integer;
begin
  Buffer := TBitMap.Create;
  try
    Buffer.Width := GraphBox.Width;
    Buffer.Height := GraphBox.Height;

    max := Buffer.Width -2;
    UnitLen := (GraphBox.Height / 2) - 8;
    UnitLenTel := 1 / UnitLen;
    Mode := WaveModeBar.Position;
    Midline := GraphBox.Height / 2;

    with Buffer.Canvas do
    begin
      Pen.Color := clBlack;
      Brush.Color := clWhite;
      Rectangle(0,0, Buffer.Width - 1, Buffer.Height -1);

      MoveTo(0, Buffer.Height div 2);
      LineTo(Buffer.Width, Buffer.Height div 2);

      Pen.Color := clRed;
      for x := 1 to max do
      begin
        ys := -CalcWave(Mode, (x - 1)*UnitLenTel) * UnitLen + MidLine;
        y := Round(ys);
        if x = 1 then
          MoveTo(x,y)
        else
          LineTo(x,y);
      end;
    end;
    GraphBox.Canvas.Draw(0,0, Buffer);

    if SaveImage then
    begin
      Buffer.SaveToFile(SavePictureDialog.FileName);
      SaveImage := False;
    end;
  finally
    Buffer.Free;
    Buffer := nil;
  end;
end;


procedure TWaveGenForm.GraphBoxDblClick(Sender: TObject);
begin
  SaveImage := SavePictureDialog.Execute;
end;

end.

